#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'GOL_VARS.py'
__create_time__ = '2020/7/15 23:15'

PUBLIC_VARS = {
    'WebDriverWait': 30,
    'poll_frequency': 0.5,
    'page_wait_timeout': 10
}