#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test03.py'
__create_time__ = '2020/7/17 2:29'

import pytest

def setup_module():
    '''测试用户文件第一个执行的方法'''

def teardown_module():
    '''测试用户文件最后一个执行的方法'''

def test_yewu_a():
    '''正常业务测试方法1'''

def test_yewu_b():
    '''正常业务测试方法2'''