#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = '__version__.py'
__create_time__ = '2020/7/15 23:21'

VERSION = (0, 3)

__version__ = '.'.join(map(str, VERSION))